#include "filter_exp.h"

//
// If this driver is going to be a filter in the paging, hibernation, or dump
// file path, then HANDLE_DEVICE_USAGE should be defined.
//
// #define HANDLE_DEVICE_USAGE 1

enum deviceState {
        STATE_INITIALIZED,
        STATE_STARTING,
        STATE_STARTED,
        STATE_START_FAILED,
        STATE_STOPPED,  // implies device was previously started successfully
        STATE_SUSPENDED,
        STATE_REMOVING,
        STATE_REMOVED
};


/*
 *  Memory tag for memory blocks allocated by this driver
 *  (used in ExAllocatePoolWithTag() call).
 *  This DWORD appears as "Filt" in a little-endian memory byte dump.
 *
 *  NOTE:  PLEASE change this value to be unique for your driver!  Otherwise,
 *  your allocations will show up with every other driver that uses 'tliF' as
 *  an allocation tag.
 *  
 */

// max 4 chars! --dvk

#define FILTER_TAG (ULONG)'_GTC'

#undef ExAllocatePool
#define ExAllocatePool(type, size) \
            ExAllocatePoolWithTag (type, size, FILTER_TAG)

typedef struct _PACKET 
{
	UCHAR data[256];
	BOOLEAN bPacketValid;
	ULONG Position; 
	ULONG Length;
	ULONG dataLength;
	ULONG Checksum;
	BOOLEAN bPacketStarted;
} PACKET, *PPACKET;


// max 4 chars! --dvk
#define DEVICE_EXTENSION_SIGNATURE '_gtC'

typedef struct _DEVICE_EXTENSION {

    /*
     *  Memory signature of a device extension, for debugging.
     */
    ULONG signature;

    /*
     *  Plug-and-play state of this device object.
     */
    enum deviceState state;

    /*
     *  The device object that this filter driver created.
     */
    PDEVICE_OBJECT filterDevObj;

    /*
     *  The device object created by the next lower driver.
     */
    PDEVICE_OBJECT physicalDevObj;

    /*
     *  The device object at the top of the stack that we attached to.
     *  This is often (but not always) the same as physicalDevObj.
     */
    PDEVICE_OBJECT topDevObj;

    /*
     *  deviceCapabilities includes a
     *  table mapping system power states to device power states.
     */
    DEVICE_CAPABILITIES deviceCapabilities;

    /*
     *  pendingActionCount is used to keep track of outstanding actions.
     *  removeEvent is used to wait until all pending actions are
     *  completed before complete the REMOVE_DEVICE IRP and let the
     *  driver get unloaded.
     */
    LONG pendingActionCount;
    KEVENT removeEvent;

	// interface name (not used yet due to installation issues)
	UNICODE_STRING ifname;

	// Interrupt handling stuff 
//nointerrupts 	PKINTERRUPT InterruptObject;

	/////////////////////////////////////////////////////////////////////
	// my private section
	/////////////////////////////////////////////////////////////////////
	
	// stuff for timer and DPC processing
	KTIMER CtgTimer;
	
	KTIMER ClrRtsTimer;
	KDPC ClrRtsDpc;

	BOOLEAN bCtgDriverIsOn;

	// IRP that is currently pending and can be safely freed/completed in OnStartWaitMaskIrpComplete()
	PIRP pPendingIrp;

// locations required for DeviceIoControl
    IO_STATUS_BLOCK ioStatusBlock;
	unsigned char pIOCtlBuffer[16];


// Receive raw data related section (data for async calls to lower drivers also)
#define RX_BUFFER_RAW_SIZE	512
#define RX_PKT_ARRAY_SIZE	4 // number of packets to be queued, increased from 3 in v.1.0.0.4
#define TX_PKT_ARRAY_SIZE	4 // number of packets to be queued

	UCHAR pRxBufferRaw[RX_BUFFER_RAW_SIZE];
	ULONG RxBufferRawWriteIndex;
	ULONG RxBufferRawReadIndex;

	PACKET arRxPkt[RX_PKT_ARRAY_SIZE]; // 4 packets max 256 bytes each and ACK packet starting at 1024
	ULONG NextRxPktRead; 
	ULONG NextRxPktWrite; 
	PPACKET pRxPktCurr;

	PACKET arTxPkt[TX_PKT_ARRAY_SIZE]; // 4 packets max 256 bytes each and ACK packet starting at 1024
	ULONG NextTxPktRead; 
	ULONG NextTxPktWrite; 
	PPACKET pTxPktCurr;

	// port status bits
	BOOLEAN bRTSIsOn; // is RTS on?
	BOOLEAN bParity; // is parity bit set?
	BOOLEAN bFirstByteJustSent; // flag for waiting until byte with 9 bit set 
	// is completely transmitted

	PACKET pktACKPacket;
	UCHAR RVUAddress; // can be 212 or (from v.1.0.0.4) 211 to 213, SET EXTERNALLY!

	// diagnostic information. Can be obtained using IOCTL_GET_DIAG_INFO request
	// *** CHANGE IOCTL_GET_DIAG_INFO HANDLING WHEN ADDING DATA HERE! ***
	DIAG_INFO diag_info;
	DRIVER_PARAMETERS driver_parameters; 

#ifdef HANDLE_DEVICE_USAGE
    /*
     *  Keep track of the number of paging/hibernation/crashdump
     *  files that are opened on this device.
     */
    ULONG  pagingFileCount, hibernationFileCount, crashdumpFileCount;
    KEVENT deviceUsageNotificationEvent;
    PVOID  pagingPathUnlockHandle;  /* handle to lock certain code as non-pageable */

    /*
     *  Also, might need to lock certain driver code as non-pageable, based on
     *  initial conditions (as opposed to paging-file considerations).
     */
    PVOID  initUnlockHandle;
    ULONG  initialFlags;
#endif // HANDLE_DEVICE_USAGE 

} DEVICE_EXTENSION, *PDEVICE_EXTENSION;

// in checked build
#if DBG
    #define DBGOUT(params_in_parentheses)   \
        {                                               \
            DbgPrint("'CEC_CTG> "); \
            DbgPrint params_in_parentheses; \
            DbgPrint("\n"); \
        }
    #define TRAP(msg)  \
        {   \
            DBGOUT(("TRAP at file %s, line %d: '%s'.", __FILE__, __LINE__, msg)); \
            DbgBreakPoint(); \
        }
#else
    #define DBGOUT(params_in_parentheses)
    #define TRAP(msg)
#endif


/*
 *  Function externs
 */
NTSTATUS    DriverEntry(IN PDRIVER_OBJECT DriverObject, IN PUNICODE_STRING RegistryPath);
NTSTATUS    Ctg_AddDevice(IN PDRIVER_OBJECT driverObj, IN PDEVICE_OBJECT pdo);
VOID        Ctg_DriverUnload(IN PDRIVER_OBJECT DriverObject);
NTSTATUS    Ctg_Dispatch(IN PDEVICE_OBJECT DeviceObject, IN PIRP Irp);
NTSTATUS    Ctg_PnP(PDEVICE_EXTENSION devExt, PIRP irp);
#ifdef HANDLE_DEVICE_USAGE
NTSTATUS    Ctg_DeviceUsageNotification(PDEVICE_EXTENSION devExt, PIRP irp);
#endif // HANDLE_DEVICE_USAGE 
NTSTATUS    Ctg_Power(PDEVICE_EXTENSION devExt, PIRP irp);
NTSTATUS    Ctg_PowerComplete(IN PDEVICE_OBJECT devObj, IN PIRP irp, IN PVOID context);
NTSTATUS    GetDeviceCapabilities(PDEVICE_EXTENSION devExt);
NTSTATUS    CallNextDriverSync(PDEVICE_EXTENSION devExt, PIRP irp);
NTSTATUS    CallDriverSync(PDEVICE_OBJECT devObj, PIRP irp);
NTSTATUS    CallDriverSyncCompletion(IN PDEVICE_OBJECT devObj, IN PIRP irp, IN PVOID Context);
VOID        IncrementPendingActionCount(PDEVICE_EXTENSION devExt);
VOID        DecrementPendingActionCount(PDEVICE_EXTENSION devExt);
NTSTATUS    QueryDeviceKey(HANDLE Handle, PWCHAR ValueNameString, PVOID Data, ULONG DataLength);
VOID        RegistryAccessSample(PDEVICE_EXTENSION devExt, PDEVICE_OBJECT devObj);


// service functions
NTSTATUS Ctg_InitStorage(PDEVICE_EXTENSION pDevExt);
VOID Ctg_InitACKPacket(IN PDEVICE_EXTENSION pDevExt, IN BOOLEAN bTestMode);

// set mask to wait (from this moment we are waiting for new event)
NTSTATUS Ctg_SetWaitMask(PDEVICE_EXTENSION pDevExt, ULONG mask);


// IRP processing 
NTSTATUS OnStartWaitMaskIrpComplete(PDEVICE_OBJECT DeviceObject, PIRP Irp, PVOID Context);
NTSTATUS OnEmptyIrpComplete(PDEVICE_OBJECT DeviceObject, PIRP Irp, PVOID Context);
NTSTATUS OnReadIrpComplete(PDEVICE_OBJECT DeviceObject, PIRP Irp, PVOID Context);
NTSTATUS OnWriteIrpComplete(PDEVICE_OBJECT DeviceObject, PIRP Irp, PVOID Context);

// Function is used for sending a comamns to serial port that returns immediately
// (like IOCTL_SERIAL_CLR_RTS or IOCTL_SERIAL_SET_RTS or IOCTL_SERIAL_SET_WAIT_MASK)
//
// Function accepts devExt->pIOCtlBuffer data
//
NTSTATUS SendIoctlSerialImmediate(PDEVICE_EXTENSION devExt, ULONG Request, ULONG nBytesToSend);
NTSTATUS StartWaitOnMask(PDEVICE_EXTENSION devExt);

// DPC-related
VOID Ctg_Dpc(IN PKDPC Dpc, IN PDEVICE_OBJECT fdo, IN PIRP junk, IN PVOID Context);

VOID Ctg_ClrRtsDpc(IN PKDPC Dpc, IN PVOID DeferredContext, IN PVOID SystemArgument1, IN PVOID SystemArgument2);

// primitives
// reads data to pRxBufferRaw 
NTSTATUS ReadPort(PDEVICE_EXTENSION pDevExt); 
// The function writes to serial port from pTxPktCurr (if exists)
NTSTATUS WritePort(PDEVICE_EXTENSION pDevExt);

NTSTATUS Ctg_SetRTS(PDEVICE_EXTENSION pDevExt);
NTSTATUS Ctg_ClrRTS(PDEVICE_EXTENSION pDevExt);
NTSTATUS Ctg_SetParityBit(PDEVICE_EXTENSION pDevExt);
NTSTATUS Ctg_ClrParityBit(PDEVICE_EXTENSION pDevExt);
NTSTATUS Ctg_SetOneStopBit(PDEVICE_EXTENSION pDevExt);
NTSTATUS Ctg_PurgeRx(PDEVICE_EXTENSION pDevExt);

// clears packet
NTSTATUS Ctg_ClearPacket(PPACKET pPkt);


/* //nointerrupts

// ISR
BOOLEAN Ctg_OnInterrupt(PKINTERRUPT InterruptObject, PVOID Context);
// DPC for ISR
VOID Ctg_DpcForIsr(PKDPC Dpc, PDEVICE_OBJECT fdo, PIRP junk, PVOID Context);

// IRP
VOID Ctg_StartIo(IN PDEVICE_OBJECT fdo, IN PIRP Irp);
VOID OnCancelActiveIrp(IN PDEVICE_OBJECT fdo, IN PIRP Irp);
BOOLEAN TransferNext(IN struct DEVICE_EXTENSION* pdx);

*/

