/*
A header file to be included into projects that use high level filter driver FILTER. 
Defines IOCTL calls to driver.
*/

#ifndef __EXPORTS_H
#define __EXPORTS_H

/*
BEFORE starting filtering you MUST set up serial port and 
set serial port timeouts to 0!
*/

// GUID to talk to driver

DEFINE_GUID(GUID_CEC_CTG, 0x3875f9ed, 0x52d6, 0x4061, 0x91, 0x14, 0x61, 0xdc, 0xa, 0xbd, 0xf6, 0xd1);


#ifndef CTL_CODE
	#pragma message("CTL_CODE undefined. Include winioctl.h or devioctl.h before this file")
#endif

// can be used to check if we are working with correct driver.
#define DRIVER_VERSION 0x00010207

// get driver version
#define IOCTL_CTG_VERSION	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x801, METHOD_BUFFERED, FILE_ANY_ACCESS)

// turn filtering on
#define IOCTL_CTG_ON	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x802, METHOD_BUFFERED, FILE_ANY_ACCESS)

// turn filtering off
#define IOCTL_CTG_OFF	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x803, METHOD_BUFFERED, FILE_ANY_ACCESS)

// get diagnostics from driver
#define IOCTL_GET_DIAG_INFO	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x804, METHOD_BUFFERED, FILE_ANY_ACCESS)

// set driver aprameters
#define IOCTL_SET_DRIVER_PARAMETERS	CTL_CODE(FILE_DEVICE_UNKNOWN, 0x805, METHOD_BUFFERED, FILE_ANY_ACCESS)

typedef struct _DIAG_INFO 
{
	ULONG nTotalPacketsReceived; // total number of packets received
	ULONG nMeaningfulPacketsReceived; // number of packets other than "57" received
	ULONG nTotalPacketsTransmitted; // total number of packets sent
	ULONG nMeaningfulPacketsTransmitted; // number of packets other than "12" sent
	ULONG TimeoutSinceLastPacketReceived; // number of timer ticks
	ULONG nPacketsInTxQueue;
	ULONG nPacketsInRxQueue;
} DIAG_INFO, *PDIAG_INFO;

// structure to set driver parameters
typedef struct _DRIVER_PARAMETERS
{
	ULONG DelayAfterRtsSet; // in microseconds, by KeStallProcessor
	ULONG DelayAfterFirstByte; // in microseconds, by KeStallProcessor
	BOOLEAN bTestMode; // 
	UCHAR RVUAddress; // externally sets RVU address answered
} DRIVER_PARAMETERS, *PDRIVER_PARAMETERS;


#endif